unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, DateUtils, Menus, Clipbrd, Strutils,
  //
  _TaskTray, EcoTrayReg,
  //
  stdFiles, stdInt, stdMedia, stdDialogs, stdStrings, Timeout,
  ExIniFiles, SizeBox,
  //
  Comps, AlarmClass, Taskbar;

type
  /// [record ݒLp]
  TTWatchConfig = record
    DateFormat      : String;
    TimeFormat      : String;
    ToolTipFormat   : String;
    InfoShowAlarm   : Boolean;
    InfoShowBeep    : Boolean;
    InfoPath        : String;
    SignalJustTime  : Boolean;
    SignalHalfTime  : Boolean;
    SignalLoopTimes : Boolean;
    SignalPath      : String;
    AlarmToBeep     : Boolean;
    AlarmSound      : String;
    NTPAddress      : String;
    AutoNTP         : Boolean;
    AutoNTPInterval : Integer;
  end;
  /// [class CEBhE NX]
  TTWatchDisplayForm = class(TForm)
    Timer: TTimer;
    PopupMenu: TPopupMenu;
    CopyDate: TMenuItem;
    CopyTime: TMenuItem;
    N1: TMenuItem;
    Options: TMenuItem;
    PropertyDateTime: TMenuItem;
    N2: TMenuItem;
    SetTime: TMenuItem;
    ShowAbout: TMenuItem;
    ShowHelp: TMenuItem;
    AppExit: TMenuItem;
    N3: TMenuItem;
    SavePosition: TMenuItem;
    AlarmSetting: TMenuItem;
    SetTopMost: TMenuItem;
    TrayIcon: TEcoTray;
    procedure FormPaint(Sender: TObject);
    procedure TimerTimer(Sender: TObject);
    procedure CopyDateClick(Sender: TObject);
    procedure CopyTimeClick(Sender: TObject);
    procedure OptionsClick(Sender: TObject);
    procedure PropertyDateTimeClick(Sender: TObject);
    procedure SetTopMostClick(Sender: TObject);
    procedure SetTimeClick(Sender: TObject);
    procedure ShowAboutClick(Sender: TObject);
    procedure AppExitClick(Sender: TObject);
    procedure SavePositionClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure AlarmSettingClick(Sender: TObject);
    procedure ShowHelpClick(Sender: TObject);
    procedure TrayIconClick(Sender: TObject; MousePos: TPoint;
      var Handled: Boolean);
  private

    { Private 錾 }
    FDisplayPanel : TTWatchPanel;
    FDateLabel    : TTWatchLabel;
    FTimeLabel    : TTWatchLabel;
    FSignalThread : TThread;
    FAlarmList    : TAlarmList;
    function GetDateText(DateTime: TDateTime): String; overload;
    function GetDateText: String; overload;
    function GetTimeText(DateTime: TDateTime): String; overload;
    function GetTimeText: String; overload;
    function GetHelpText: String;
    procedure ReloadSetting(Ini: TMemIniFileEx); overload;
    procedure ReloadSetting(); overload;
    procedure ReloadAlarm();
    procedure SetWindowRect();
    procedure SetStatusText(Text: String; InfoAlarm: Boolean = True);
    procedure SetAutoNTP(SetTime: Boolean);
  protected
    { Protected 錾 }
    procedure CreateParams(var Params: TCreateParams); override;
    procedure CustomAlignPosition(Control: TControl; var NewLeft: Integer;
      var NewTop: Integer; var NewWidth: Integer; var NewHeight: Integer;
      var AlignRect: TRect; AlignInfo: TAlignInfo); override;
    procedure CMShowingchanged(var Message: TMessage);
      message CM_SHOWINGCHANGED;

  public
    { Public 錾 }
    Setting    : TTWatchConfig;
    class function  GetFormattedText(fmt: String; DateTime: TDateTime): String;
    property  DisplayPanel: TTWatchPanel      read  FDisplayPanel;
    property  TimeLabel   : TTWatchLabel      read  FTimeLabel;
    property  DateLabel   : TTWatchLabel      read  FDateLabel;
  end;

var
  TWatchDisplayForm: TTWatchDisplayForm;
const
  TWatchExtendFormats : array[0..5] of String =
    ('%tch', '%tcn', '%tcs', '%tcm', '%tcms', '%n');
  DT_MARGIN   = 6;
  FRAME_SIZE  = 2;
  // ^C}[LParam
  TIMER_STATUS= 1;
  TIMER_ALARM = 2;
  TIMER_SIGNAL= 3;
  // 萔
  ONE_MINUTE  = 60000;
  ONE_SECOND  = 1000;
  HIDE_STATUS = ONE_SECOND * 5;
implementation

uses Math, SNTP, Types, IniFiles, PreferenceDlg, StdCtrls, ChoiLabels,
  AboutDialog, AlarmDlg;

{$R *.dfm}

{ TTWatchDisplayForm }

(* I[o[Ch\bh *)

procedure TTWatchDisplayForm.CreateParams(var Params: TCreateParams);
begin
  inherited;
  Params.ExStyle := Params.ExStyle or WS_EX_TOOLWINDOW;
  if SetTopMost.Checked then
    Params.ExStyle := Params.ExStyle or WS_EX_TOPMOST;
end;

procedure TTWatchDisplayForm.CMShowingchanged(var Message: TMessage);
begin
  if SetTopMost.Checked then
    TopMost(WS_EX_TOPMOST);
  inherited;
end;

(* tH[Cxg *)

procedure TTWatchDisplayForm.FormCreate(Sender: TObject);
var
  ini : TMemIniFileEx;
  l   : array[0..1] of TTWatchLabel;
  i   : Integer;
begin
{$IFDEF VER150}
  ScreenSnap     := True;
{$ENDIF}

  FDisplayPanel       := TTWatchPanel.Create(Self);
  DisplayPanel.Parent := Self;
  DisplayPanel.Align  := alCustom;
  DisplayPanel.DoubleBuffered := true;
  FTimeLabel          := TTWatchLabel.Create(DisplayPanel);
  FDateLabel          := TTWatchLabel.Create(DisplayPanel);
  TimeLabel.Align     := alClient;
  DateLabel.Align     := alBottom;
  l[0] := TimeLabel; l[1] := DateLabel;
  for i := 0 to 1 do
  with l[i] do begin
    Alignment         := taCenter;
    Transparent       := True;
    Layout            := tlCenter;
    Parent            := DisplayPanel;
    Shadow.X          := 2;
    Shadow.Y          := 2;
  end;
  FAlarmList          := TAlarmList.Create;
  // ݒǂݍ

  ini := TMemIniFileEx.CreateDef;
  try
    ReloadSetting(ini);
    ini.Section       := 'General';
    with ini.ReadPointS('Point', Point(Self.Left, Self.Top)) do begin
      Self.Left       := X;
      Self.Top        := Y;
    end;
    if ini.ReadBoolS('TopMostMode', False) then
      SetTopMost.Click;
  finally
    ini.Free;
  end;
  ReloadAlarm;
end;

procedure TTWatchDisplayForm.FormDestroy(Sender: TObject);
begin
  Timer.Enabled := False;
  if Assigned(FSignalThread) then
    FSignalThread.Terminate;
  FAlarmList.Free;
end;

procedure TTWatchDisplayForm.FormPaint(Sender: TObject);
var rc: TRect;
begin
  rc := ClientRect;
  Frame3D(Canvas, rc,
    EnsureRange(ColorToRGB(Self.Color) + $202020, clBlack, clWhite),
    EnsureRange(ColorToRGB(Self.Color) - $202020, clBlack, clWhite),
    FRAME_SIZE);
end;

procedure TTWatchDisplayForm.CustomAlignPosition(Control: TControl;
  var NewLeft, NewTop, NewWidth, NewHeight: Integer; var AlignRect: TRect;
  AlignInfo: TAlignInfo);
begin
  inherited;
  NewLeft   := FRAME_SIZE;
  NewTop    := FRAME_SIZE;
  NewHeight := ClientHeight - FRAME_SIZE * 2;
  NewWidth  := ClientWidth - FRAME_SIZE * 2;
end;

(* Timer *)

procedure TTWatchDisplayForm.TimerTimer(Sender: TObject);
  procedure TimeSignal();
  var h, m, s, ms: Word;
  begin
    DecodeTime(Now, h, m, s, ms);
    // 00
    if Setting.SignalJustTime and (m = 0) then begin
      if Setting.SignalLoopTimes then
        FSignalThread :=
          DupeSounds(Setting.SignalPath, IfThen(h = 0, 12, h mod 12))
      else
        snd(Setting.SignalPath);
    end;
    // 30
    if Setting.SignalHalfTime and (m = 30) then begin
      snd(Setting.SignalPath);
    end;
    setTimeout(nil, ONE_MINUTE, TIMER_SIGNAL); // dĐh~^C}[
  end;

  procedure CheckAlarm();
  var i : Integer;
  begin
    for i := 0 to FAlarmList.Count - 1 do begin
      if FAlarmList[i].IsAlarmTime(Now) then begin
        // A[
        if Setting.AlarmToBeep then
          Windows.Beep(1000, 100)
        else
          snd(Setting.AlarmSound);
        SetStatusText(FAlarmList[i].Message, False);
        // R}hgp
        if FAlarmList[i].Command <> '' then
          Shell(FAlarmList[i].Command);
        // KvȂ폜ăZ[u
        if FAlarmList[i].RemoveOnExec then begin
          FAlarmList.Delete(i);
          FAlarmList.SaveToFile;
        end;
        setTimeout(nil, ONE_MINUTE, TIMER_ALARM);
      end;
    end;
  end;

begin
  TimeLabel.Caption := GetTimeText;
  Hint              := GetHelpText;
  TrayIcon.Tip      := Hint;
  // A[`FbN
  if not IsactiveTimeoutlParam(TIMER_ALARM) then CheckAlarm;
  // 炷
  if not IsactiveTimeoutlParam(TIMER_SIGNAL) and (MinuteOf(Now) mod 30 = 0) then
    TimeSignal;
  // 킹`FbN
  if (Timer.Tag <> 0) and (GetTickCount >= Cardinal(Timer.Tag)) then SetAutoNTP(True);
  // Xe[^X͕\łȂ΁At̕\
  if not IsactiveTimeoutlParam(TIMER_STATUS) then
    DateLabel.Caption := GetDateText;
end;

(* TrayIcon *)

procedure TTWatchDisplayForm.TrayIconClick(Sender: TObject;
  MousePos: TPoint; var Handled: Boolean);
begin
  if not Visible then
    Syssnd('RestoreUp')
  else
    Syssnd('Minimize');
  Visible := not Visible;
  if Visible then SetForegroundWindow(Handle);
  Handled := True;
end;

(* \bh *)

procedure TTWatchDisplayForm.ReloadSetting(Ini: TMemIniFileEx);
var
  O: TClass;
begin
  with ini do begin
    Section := 'General';
    Setting.DateFormat        := ReadStringS('DateFormat', 'yyyy/mm/dd');
    Setting.TimeFormat        := ReadStringS('TimeFormat', 'ampm h:nn:ss');
    Setting.ToolTipFormat     := ReadStringS('ToolTipFormat', '');
    Setting.NTPAddress        := ReadStringS('NTPAddress', 'ntp1.jst.mfeed.ad.jp');
    Self.AlphaBlendValue      := ReadIntegerS('Transparency', 255);
    Self.AlphaBlend           := Self.AlphaBlendValue <> 255;
    Self.Color                := ReadColorS('BackColor', clBtnFace);
    O := ExceptionClass;
    ExceptionClass := EAbort;
    try
      try
        DisplayPanel.Background.LoadFromFile(ReadPathS('BGImage'));
      except
      end;
    finally
      ExceptionClass := O
    end;
    Setting.AutoNTP           := ReadBoolS('AutoNTP', False);
    Setting.AutoNTPInterval := ReadIntegerS('AutoNTPInterval', 10);
    SetAutoNTP(False);
    Section := 'Font';
    Font.Name                 := ReadStringS('FontName', Self.Font.Name);
    DateLabel.Font.Assign(Self.Font);
    DateLabel.Font.Size       := ReadIntegerS('DateSize', Self.Font.Size * 2);
    DateLabel.Font.Color      := ReadColorS('DateColor', clWindowText);
    DateLabel.Shadow.Enabled  := ReadBoolS('DateShadow');
    DateLabel.Shadow.Color    := ReadColorS('DateShadowColor', clSilver);
    TimeLabel.Font.Assign(Self.Font);
    TimeLabel.Font.Size       := ReadIntegerS('TimeSize', Self.Font.Size * 2);
    TimeLabel.Font.Color      := ReadColorS('TimeColor', clWindowText);
    TimeLabel.Shadow.Enabled  := ReadBoolS('TimeShadow');
    TimeLabel.Shadow.Color    := ReadColorS('TimeShadowColor', clSilver);
    Section := 'InformationNote';
    Setting.InfoShowAlarm     := ReadBoolS('Play', False);
    Setting.InfoShowBeep      := ReadBoolS('OnBeep', False);
    Setting.InfoPath          := ReadPathS('Path', '');
    Section := 'TimeSignal';
    Setting.SignalJustTime    := ReadBoolS('PlayJustTime');
    Setting.SignalHalfTime    := ReadBoolS('PlayHalfTime');
    Setting.SignalLoopTimes   := ReadBoolS('TimesLoop');
    Setting.SignalPath        := ReadPathS('Path');
    Section := 'Alarm';
    Setting.AlarmToBeep       := ReadBoolS('AlarmToBeep');
    Setting.AlarmSound        := ReadPathS('Path');
  end;
  SetWindowRect;

end;

procedure TTWatchDisplayForm.ReloadSetting;
var
  ini: TMemIniFileEx;
begin
  ini := TMemIniFileEx.CreateDef;
  try
    ReloadSetting(ini);
  finally
    ini.Free;
  end;
end;

procedure TTWatchDisplayForm.ReloadAlarm;
begin
  FAlarmList.LoadFromFile;
end;

procedure TTWatchDisplayForm.SetWindowRect;
var TimeRect, DateRect: TRect;
begin
  // Ԍv
  FillChar(TimeRect, SizeOf(TRect), #0);
  DrawText(TimeLabel.Canvas.Handle, PChar(GetTimeText(EndOfTheDay(Today))), -1,
    TimeRect, DT_CALCRECT or DT_NOPREFIX);
  // tv
  FillChar(DateRect, SizeOf(TRect), #0);
  DrawText(DateLabel.Canvas.Handle, PChar(GetDateText(EndOfTheYear(Today))), -1,
    DateRect, DT_CALCRECT or DT_NOPREFIX);

  // TCYݒ
  DateLabel.Height := DateRect.Bottom;
  Self.ClientWidth :=
    Max(DateRect.Right, TimeRect.Right);
  Self.ClientHeight:=
    DateRect.Bottom + TimeRect.Bottom + DT_MARGIN;
end;

procedure TTWatchDisplayForm.SetStatusText(Text: String; InfoAlarm: Boolean = True);
begin
  DateLabel.Caption := Text;
  if InfoAlarm and Setting.InfoShowAlarm then begin
    if Setting.InfoShowBeep then
      Windows.Beep(900, 300)
    else
      snd(Setting.InfoPath);
  end;
  // \\莞Zbg
  setTimeout(nil, 5000, TIMER_STATUS);
end;

procedure TTWatchDisplayForm.SetAutoNTP(SetTime: Boolean);
begin
  if SetTime then begin
    SetStatusText(Format('킹%s܂',
    [IfThen(NTPSetTime(Setting.NTPAddress), '', 's')]));
  end;
  if Setting.AutoNTP then begin
    // ̎ݒ
    Timer.Tag := IfThen(Setting.AutoNTP,
      Integer(GetTickCount) + Setting.AutoNTPInterval * 60 * 1000, 0);
  end;
end;

function TTWatchDisplayForm.GetDateText(DateTime: TDateTime): String;
begin
  Result := GetFormattedText(Setting.DateFormat, DateTime);
end;

function TTWatchDisplayForm.GetDateText: String;
begin
  Result := GetDateText(Today);
end;

function TTWatchDisplayForm.GetTimeText(DateTime: TDateTime): String;
begin
  Result := GetFormattedText(Setting.TimeFormat, DateTime);
end;

function TTWatchDisplayForm.GetTimeText: String;
begin
  Result := GetTimeText(Now);
end;

function TTWatchDisplayForm.GetHelpText: String;
begin
  Result := GetFormattedText(Setting.ToolTipFormat, Now);
end;

(* NX\bh *)

class function TTWatchDisplayForm.GetFormattedText(fmt: String; DateTime: TDateTime): String;
var h, m, s, ms: Word;
begin
  DivideMSec(GetTickCount, h, m, s, ms);
  fmt := StringsReplace(
    fmt, TWatchExtendFormats,
      [Format('%d', [h]),
      Format('%.2d', [m]),
      Format('%.2d', [s]),
      Format('%.3d', [ms]),
      Format('%.3d', [GetTickCount]),
      #13#10],
      [rfReplaceAll]
      );
  Result := FormatDateTime(fmt, DateTime);
end;

(* j[ *)

procedure TTWatchDisplayForm.AlarmSettingClick(Sender: TObject);
begin
  if ShowDialog(TTWatchAlarmDialog) = mrOk then
    ReloadAlarm;
end;

procedure TTWatchDisplayForm.CopyDateClick(Sender: TObject);
begin
  Clipboard.AsText := GetDateText;
end;

procedure TTWatchDisplayForm.CopyTimeClick(Sender: TObject);
begin
  Clipboard.AsText := GetTimeText;
end;

procedure TTWatchDisplayForm.OptionsClick(Sender: TObject);
begin
  if ShowDialog(TTWatchPreferenceDialog) = mrOk then
    ReloadSetting;
end;

procedure TTWatchDisplayForm.PropertyDateTimeClick(Sender: TObject);
begin
  RunAs('control', 'timedate.cpl')
end;

procedure TTWatchDisplayForm.SetTopMostClick(Sender: TObject);
begin
  if SetTopMost.Checked  then
    TopMost(Handle)
  else
    UnTopMost(Handle);
end;

procedure TTWatchDisplayForm.SavePositionClick(Sender: TObject);
begin
  with TMemIniFileEx.CreateDef do
  try
    Section := 'General';
    WritePointS('Point', Self.BoundsRect.TopLeft);
    WriteBoolS('TopMostMode', FormStyle = fsStayOnTop);
    UpdateFile;
  finally
    Free;
  end;
end;

procedure TTWatchDisplayForm.SetTimeClick(Sender: TObject);
begin
  SetAutoNTP(True);
end;

procedure TTWatchDisplayForm.ShowAboutClick(Sender: TObject);
begin
  ShowDialog(TTWatchAboutDialog);
end;

procedure TTWatchDisplayForm.ShowHelpClick(Sender: TObject);
begin
  Application.HelpCommand(HELP_CONTENTS, 0);
end;

procedure TTWatchDisplayForm.AppExitClick(Sender: TObject);
begin
  Self.Close;
end;


end.

