unit AlarmClass;

interface
uses
  Classes, TypInfo, Windows, Controls, MMSystem, Contnrs, IniFiles,
  //
  stdInt, ExIniFiles;
type
  TAlarmKind  = (akAtWeeks, akAtDate, akAtTickCount);
  TSnoozeType = (snDisable, sn5Minutes, sn10Minutes, sn15Minutes);
  TWeek       = (wSun, wMon, wTue, wWed, wThu, wFri, wSat);
  TWeeks      = set of TWeek;
  TAlarmData  = class(TPersistent)
  private
    { Private 錾 }
    FAlarmKind    : TAlarmKind;
    FDate         : TDate;
    FTime         : TTime;
    FWeeks        : TWeeks;
    FSnooze       : TSnoozeType;
    FMessage      : String;
    FCommand      : String;
    FRemoveOnExec : Boolean;
  protected
    { Protected 錾 }
  public
    { Public 錾 }
    constructor Create(); overload;
    constructor Create(Ini: TCustomIniFile; Section: String); overload;
    destructor Destroy; override;

    function  IsAlarmTime(DateTime: TDateTime): Boolean;
    function  ToString(): String;
    class function AllDay(): TWeeks;
    procedure SaveIniFile(Ini: TCustomIniFile; Section: String);
    procedure LoadIniFile(Ini: TCustomIniFile; Section: String);
  published
    { Published 錾 }
    property  AlarmKind   : TAlarmKind  read  FAlarmKind    write FAlarmKind  default akAtWeeks;
    property  Date        : TDate       read  FDate         write FDate;
    property  Time        : TTime       read  FTime         write FTime;
    property  Weeks       : TWeeks      read  FWeeks        write FWeeks;
    property  Snooze      : TSnoozeType read  FSnooze       write FSnooze;
    property  Message     : String      read  FMessage      write FMessage;
    property  Command     : String      read  FCommand      write FCommand;
    property  RemoveOnExec: Boolean     read  FRemoveOnExec write FRemoveOnExec;
  end;

  TAlarmList = class(TObjectList)
  private
    { Private 錾 }
    function GetItem(Index: Integer): TAlarmData;
    procedure SetItem(Index: Integer; const Value: TAlarmData);
  public
    { Public 錾 }
    procedure SaveToFile();
    procedure LoadFromFile();
    function  Add(AObject: TAlarmData): Integer;
    function  Extract(Item: TAlarmData): TAlarmData;
    function  First: TAlarmData;
    function  IndexOf(AObject: TAlarmData): Integer;
    procedure Insert(Index: Integer; AObject: TAlarmData);
    function  Last: TAlarmData;
    function  Remove(AObject: TAlarmData): Integer;
    property  Items[Index: Integer] : TAlarmData read  GetItem write SetItem; default;
  end;

implementation

uses DateUtils, SysUtils, StrUtils, Main;

{ TAlarmData }

class function TAlarmData.AllDay: TWeeks;
begin
  Result := [wSun, wMon, wTue, wWed, wThu, wFri, wSat];
end;

constructor TAlarmData.Create;
begin
  inherited;
  FAlarmKind := akAtWeeks;
end;

constructor TAlarmData.Create(Ini: TCustomIniFile; Section: String);
begin
  inherited Create();
  LoadIniFile(Ini, Section);
end;

destructor TAlarmData.Destroy;
begin
  inherited;
end;

function TAlarmData.IsAlarmTime(DateTime: TDateTime): Boolean;
  // Ԃƕ}b`Ă邩ǂԂ
  function MatchTime(): Boolean;
  const
    SnoozeTimes: array[TSnoozeType] of Integer = (0, 5, 10, 15);
  var
    i           : Integer;
    b           : Boolean;
    t           : TTime;
    h, m, s, ms : Word;
  begin
    // bƃ~bJbg
    DecodeTime(DateTime, h, m, s, ms);
    t := EncodeTime(h, m, 0, 0);
    if Snooze <> snDisable then begin
      // Xk[Y
      b     := False;
      for i := 1 to 4 do
        b   := b or (t = IncMinute(Time, SnoozeTimes[Snooze] * i));
      Result:= (t = Time) or b;
    end else
      // Xk[Y
      Result:= (t = Time);
  end;
begin
  case AlarmKind of
  akAtWeeks     : // jw/
    Result        := (TWeek(DayOfWeek(DateTime) - 1) in Weeks) and
      MatchTime;
  akAtDate      : // t
    Result        := (DateOf(DateTime) = Date) and MatchTime;
  else Result     := False;
  end;

end;

procedure TAlarmData.SaveIniFile(Ini: TCustomIniFile; Section: String);
  function GetString(PropName: String): String;
  var
    Ident     : String;
    IntToIdent: TIntToIdent;
  begin
    case PropType(Self, PropName) of
    tkInteger:
      begin
        IntToIdent := FindIntToIdent(GetPropInfo(Self, PropName).PropType^);
        if Assigned(IntToIdent) and IntToIdent(GetPropValue(Self, PropName), Ident) then
          Result := Ident
        else
          Result := GetPropValue(Self, PropName);
      end;
    tkChar, tkEnumeration, tkSet, tkFloat, tkString, tkVariant, tkInt64:
      Result := GetPropValue(Self, PropName);
    end;
  end;
begin
  Ini.WriteString(Section, 'AlarmKind', GetString('AlarmKind'));
  Ini.WriteDate(Section, 'Date', Date);
  Ini.WriteTime(Section, 'Time', Time);
  Ini.WriteString(Section, 'Weeks', GetString('Weeks'));
  Ini.WriteString(Section, 'Snooze', GetString('Snooze'));
  Ini.WriteString(Section, 'Message', Message);
  Ini.WriteString(Section, 'Command', Command);
  Ini.WriteBool(Section, 'RemoveOnExec', RemoveOnExec);
end;

procedure TAlarmData.LoadIniFile(Ini: TCustomIniFile; Section: String);
  // http://leed.issp.u-tokyo.ac.jp/~takeuchi/delphi/browse.cgi?index=75958
  procedure SetString(PropName, Value: String);
  var
    i         : Integer;
    IdentToInt: TIdentToInt;
  begin
    case PropType(Self, PropName) of
    tkInteger:
      begin
        IdentToInt:= FindIdentToInt(GetPropInfo(Self, PropName).PropType^);
        if Assigned(IdentToInt) and IdentToInt(Value, i) then
          SetPropValue(Self, PropName, i)
        else
          SetPropValue(Self, PropName, Value);
      end;
    tkChar, tkEnumeration, tkSet, tkFloat, tkString, tkVariant, tkInt64:
      SetPropValue(Self, PropName, Value);
    end;
  end;
begin
  SetString('AlarmKind', Ini.ReadString(Section, 'AlermKind', 'akAtWeeks'));
  Date        := Ini.ReadDate(Section, 'Date', Today);
  Time        := Ini.ReadTime(Section, 'Time', Now);
  SetString('Weeks', Ini.ReadString(Section, 'Weeks', '[]'));
  SetString('Snooze', Ini.ReadString(Section, 'Snooze', 'snDisable'));
  Message     := Ini.ReadString(Section, 'Message', '');
  Command     := Ini.ReadString(Section, 'Command', Command);
  RemoveOnExec:= Ini.ReadBool(Section, 'RemoveOnExec', False);
end;

function TAlarmData.ToString: String;
var
  t: array[TAlarmKind] of String;
  function GetWeekStr(): String;
  const
    WeekLabel: array[TWeek] of String = (
      '', '', '', '', '', '', 'y');
  var
    i : Integer;
  begin
    Result := '';
    for i := 0 to Length(WeekLabel) - 1 do
      if TWeek(i) in Weeks then
        Result := Result + WeekLabel[TWeek(i)];
  end;

begin
  t[akAtWeeks]    := IfThen(Weeks = allDay, '', GetWeekStr) + ' ' +
    FormatDateTime('h:mm', Time);
  t[akAtDate]     := FormatDateTime('m/dd', Date) + ' ' +
    FormatDateTime('h:nn', Time);

  Result          :=
    Format('%s - %s', [IfThen(Message = '', '', Message), t[AlarmKind]]);
end;

{ TAlarmList }

function TAlarmList.Add(AObject: TAlarmData): Integer;
begin
  Result := inherited Add(AObject);
end;

function TAlarmList.Extract(Item: TAlarmData): TAlarmData;
begin
  Result := TAlarmData(inherited Extract(Item));
end;

function TAlarmList.First: TAlarmData;
begin
  Result := TAlarmData(inherited First);
end;

function TAlarmList.IndexOf(AObject: TAlarmData): Integer;
begin
  Result := inherited IndexOf(AObject);
end;

procedure TAlarmList.Insert(Index: Integer; AObject: TAlarmData);
begin
  inherited Insert(Index, AObject);
end;

function TAlarmList.Last: TAlarmData;
begin
  Result := TAlarmData(inherited Last);
end;

function TAlarmList.Remove(AObject: TAlarmData): Integer;
begin
  Result := inherited Remove(AObject);
end;

function TAlarmList.GetItem(Index: Integer): TAlarmData;
begin
  Result := TAlarmData(inherited GetItem(Index));
end;

procedure TAlarmList.SetItem(Index: Integer; const Value: TAlarmData);
begin
  inherited SetItem(Index, Value);
end;

procedure TAlarmList.LoadFromFile;
var
  i   : Integer;
  ini : TMemIniFileEx;
  secs: TStringList;
begin
  Clear;
  ini := TMemIniFileEx.CreateExpand('%APPLICATION%\Alarm.ini');
  try
    secs:= TStringList.Create;
    try
      ini.ReadSections(secs);
      for i := 0 to secs.Count - 1 do
        Add(TAlarmData.Create(ini, secs[i]));
    finally
      secs.Free;
    end;
  finally
    ini.Free;
  end;
end;

procedure TAlarmList.SaveToFile;
var
  i   : Integer;
  ini : TMemIniFileEx;
begin
  ini := TMemIniFileEx.CreateExpand('%APPLICATION%\Alarm.ini');
  try
    ini.Clear;
    for i := 0 to Count - 1 do
      Items[i].SaveIniFile(ini, Format('Alarm%d', [i + 1]));
    ini.UpdateFile;
  finally
    ini.Free;
  end;
end;

end.

